from snips_nlu.cli.download import download
from snips_nlu.cli.generate_dataset import generate_dataset
from snips_nlu.cli.inference import parse
from snips_nlu.cli.link import link
from snips_nlu.cli.metrics import train_test_metrics, cross_val_metrics
from snips_nlu.cli.training import train



# coding=utf-8
from __future__ import unicode_literals, print_function

from pathlib import Path

from snips_nlu.cli.dataset.entities import CustomEntity, create_entity
from snips_nlu.cli.dataset.intent_dataset import IntentDataset


class AssistantDataset(object):
    """Dataset of an assistant

    Merges a list of :class:`.AssistantDataset` into a single dataset ready to
    be used by Snips NLU

    Attributes:
        language (str): language of the assistant
        intents_datasets (list of :class:`.IntentDataset`): data of the
            assistant intents
        entities (list of :class:`.Entity`): data of the assistant entities
    """

    def __init__(self, language, intent_datasets, entities):
        self.language = language
        self.intents_datasets = intent_datasets
        self.entities = entities

    @classmethod
    def from_files(cls, language, filenames):
        """Creates an :class:`.AssistantDataset` from a language and a list of
        intent and entity files

        Args:
            language (str): language of the assistant
            filenames (list of str): Intent and entity files.
                The assistant will associate each intent file to an intent,
                and each entity file to an entity. For instance, the intent
                file 'intent_setTemperature.txt' will correspond to the intent
                'setTemperature', and the entity file 'entity_room.txt' will
                correspond to the entity 'room'.
        """
        intent_filepaths = set()
        entity_filepaths = set()
        for filename in filenames:
            filepath = Path(filename)
            stem = filepath.stem
            if stem.startswith("intent_"):
                intent_filepaths.add(filepath)
            elif stem.startswith("entity_"):
                entity_filepaths.add(filepath)
            else:
                raise AssertionError("Filename should start either with "
                                     "'intent_' or 'entity_' but found: %s"
                                     % stem)

        intents_datasets = [IntentDataset.from_file(f)
                            for f in intent_filepaths]

        entities = [CustomEntity.from_file(f) for f in entity_filepaths]
        entity_names = set(e.name for e in entities)

        # Add entities appearing only in the intents data
        for intent_data in intents_datasets:
            for entity_name in intent_data.entities_names:
                if entity_name not in entity_names:
                    entity_names.add(entity_name)
                    entities.append(create_entity(entity_name))
        return cls(language, intents_datasets, entities)

    @property
    def json(self):
        intents = {intent_data.intent_name: intent_data.json
                   for intent_data in self.intents_datasets}
        entities = {entity.name: entity.json for entity in self.entities}
        return dict(language=self.language, intents=intents, entities=entities)



from __future__ import unicode_literals

import json
from builtins import next
from pathlib import Path

from snips_nlu_utils import normalize

from snips_nlu.constants import (STOP_WORDS, WORD_CLUSTERS, GAZETTEERS, NOISE,
                                 STEMS, DATA_PATH)
from snips_nlu.languages import get_default_sep
from snips_nlu.tokenization import tokenize
from snips_nlu.utils import is_package, get_package_path

_RESOURCES = dict()


class MissingResource(LookupError):
    pass


def clear_resources():
    _RESOURCES.clear()


def load_resources(name):
    """Load language specific resources

    Args:
        name (str): resource name

    Note:
        Language resources must be loaded before fitting or parsing
    """
    if name in set(d.name for d in DATA_PATH.iterdir()):
        _load_resources_from_dir(DATA_PATH / name)
    elif is_package(name):
        package_path = get_package_path(name)
        _load_resources_from_dir(package_path)
    elif Path(name).exists():
        _load_resources_from_dir(Path(name))
    else:
        raise MissingResource("Language resource '{r}' not found. This may be "
                              "solved by running 'snips-nlu download {r}'"
                              .format(r=name))


def resource_exists(language, resource_name):
    """Tell if the resource specified by the resource_name exist

        Args:
            language (str): language
            resource_name (str): the resource name
        Returns:
            bool: whether the resource exists or not
    """
    return resource_name in _RESOURCES[language] \
           and _RESOURCES[language][resource_name] is not None


def get_stop_words(language):
    return _get_resource(language, STOP_WORDS)


def get_noise(language):
    return _get_resource(language, NOISE)


def get_word_clusters(language):
    return _get_resource(language, WORD_CLUSTERS)


def get_word_cluster(language, cluster_name):
    word_clusters = get_word_clusters(language)
    if cluster_name not in word_clusters:
        raise MissingResource("Word cluster '{}' not found for language '{}'"
                              .format(cluster_name, language))
    return word_clusters[cluster_name]


def get_gazetteer(language, gazetteer_name):
    gazetteers = _get_resource(language, GAZETTEERS)
    if gazetteer_name not in gazetteers:
        raise MissingResource("Gazetteer '{}' not found for language '{}'"
                              .format(gazetteer_name, language))
    return gazetteers[gazetteer_name]


def get_stems(language):
    return _get_resource(language, STEMS)


def _get_resource(language, resource_name):
    if language not in _RESOURCES:
        raise MissingResource(
            "Missing resources for language '%s', please load them with the "
            "load_resources function" % language)
    if resource_name not in _RESOURCES[language] \
            or _RESOURCES[language][resource_name] is None:
        raise MissingResource("Resource '{}' not found for language '{}'"
                              .format(resource_name, language))
    return _RESOURCES[language][resource_name]


def _load_resources_from_dir(resources_dir):
    with (resources_dir / "metadata.json").open() as f:
        metadata = json.load(f)
    language = metadata["language"]
    resource_name = metadata["name"]
    version = metadata["version"]
    if language in _RESOURCES:
        return
    sub_dir = resources_dir / (resource_name + "-" + version)
    if not sub_dir.is_dir():
        raise FileNotFoundError("Missing resources directory: %s"
                                % str(sub_dir))
    word_clusters = _load_word_clusters(sub_dir / "word_clusters")
    gazetteers = _load_gazetteers(sub_dir / "gazetteers", language)
    stop_words = _load_stop_words(sub_dir / "stop_words.txt")
    noise = _load_noise(sub_dir / "noise.txt")
    stems = _load_stems(sub_dir / "stemming")

    _RESOURCES[language] = {
        WORD_CLUSTERS: word_clusters,
        GAZETTEERS: gazetteers,
        STOP_WORDS: stop_words,
        NOISE: noise,
        STEMS: stems
    }


def _load_stop_words(stop_words_path):
    if not stop_words_path.exists():
        return None
    with stop_words_path.open(encoding='utf8') as f:
        lines = (normalize(l) for l in f)
        stop_words = set(l for l in lines if l)
    return stop_words


def _load_noise(noise_path):
    if not noise_path.exists():
        return None
    with noise_path.open(encoding='utf8') as f:
        # Here we split on a " " knowing that it's always ignored by
        # the tokenization (see tokenization unit tests)
        # It is not important to tokenize precisely as this noise is just used
        # to generate utterances for the None intent
        noise = next(f).split()
    return noise


def _load_word_clusters(word_clusters_path):
    if not word_clusters_path.is_dir():
        return dict()

    clusters = dict()
    for filepath in word_clusters_path.iterdir():
        word_cluster_name = filepath.stem
        with filepath.open(encoding="utf8") as f:
            clusters[word_cluster_name] = dict()
            for line in f:
                split = line.rstrip().split("\t")
                if len(split) == 2:
                    clusters[word_cluster_name][split[0]] = split[1]
    return clusters


def _load_gazetteers(gazetteers_path, language):
    if not gazetteers_path.is_dir():
        return dict()

    gazetteers = dict()
    for filepath in gazetteers_path.iterdir():
        gazetteer_name = filepath.stem
        with filepath.open(encoding="utf8") as f:
            gazetteers[gazetteer_name] = set()
            for line in f:
                normalized = normalize(line.strip())
                if normalized:
                    token_values = (t.value
                                    for t in tokenize(normalized, language))
                    normalized = get_default_sep(language).join(token_values)
                    gazetteers[gazetteer_name].add(normalized)
    return gazetteers


def _load_verbs_lexemes(stemming_path):
    try:
        lexems_path = next(stemming_path.glob("top_*_verbs_lexemes.txt"))
    except StopIteration:
        return None

    verb_lexemes = dict()
    with lexems_path.open(encoding="utf8") as f:
        for line in f:
            elements = line.strip().split(';')
            verb = normalize(elements[0])
            lexemes = elements[1].split(',')
            verb_lexemes.update(
                {normalize(lexeme): verb for lexeme in lexemes})
    return verb_lexemes


def _load_words_inflections(stemming_path):
    try:
        inflection_path = next(stemming_path.glob("top_*_words_inflected.txt"))
    except StopIteration:
        return None

    inflections = dict()
    with inflection_path.open(encoding="utf8") as f:
        for line in f:
            elements = line.strip().split(';')
            inflections[normalize(elements[0])] = normalize(elements[1])
    return inflections


def _load_stems(stemming_path):
    inflexions = _load_words_inflections(stemming_path)
    lexemes = _load_verbs_lexemes(stemming_path)
    stems = None
    if inflexions is not None:
        stems = inflexions
    if lexemes is not None:
        if stems is None:
            stems = dict()
        stems.update(lexemes)
    return stems



from __future__ import unicode_literals

import json
from builtins import bytes
from pathlib import Path

import plac

from snips_nlu import load_resources, SnipsNLUEngine


@plac.annotations(
    dataset_path=("Path to the training dataset file", "positional", None,
                  str),
    output_path=("Path of the output model", "positional", None, str),
    config_path=("Path to the NLU engine configuration", "option", "c", str))
def train(dataset_path, output_path, config_path):
    """Train an NLU engine on the provided dataset"""
    with Path(dataset_path).open("r", encoding="utf8") as f:
        dataset = json.load(f)

    config = None
    if config_path is not None:
        with Path(config_path).open("r", encoding="utf8") as f:
            config = json.load(f)

    load_resources(dataset["language"])
    engine = SnipsNLUEngine(config).fit(dataset)
    print("Create and train the engine...")

    serialized_engine = bytes(json.dumps(engine.to_dict()), encoding="utf8")
    with Path(output_path).open("w", encoding="utf8") as f:
        f.write(serialized_engine.decode("utf8"))
    print("Saved the trained engine to %s" % output_path)



# inspired from:
# https://python-packaging-user-guide.readthedocs.io/guides/single-sourcing-package-version/
# https://github.com/pypa/warehouse/blob/master/warehouse/__about__.py

# pylint:disable=line-too-long

__title__ = "snips_nlu"
__summary__ = "Snips Natural Language Understanding library"
__uri__ = "https://snips-nlu.readthedocs.io"
__author__ = "Clement Doumouro, Adrien Ball"
__email__ = "clement.doumouro@snips.ai, adrien.ball@snips.ai"
__license__ = "Apache License, Version 2.0"

__version__ = "0.15.1"
__model_version__ = "0.15.0"

__download_url__ = "https://github.com/snipsco/snips-nlu-language-resources/releases/download"
__compatibility__ = "https://raw.githubusercontent.com/snipsco/snips-nlu-language-resources/master/compatibility.json"
__shortcuts__ = "https://raw.githubusercontent.com/snipsco/snips-nlu-language-resources/master/shortcuts.json"

# pylint:enable=line-too-long



from __future__ import print_function, unicode_literals

import sys

import requests


def prints(*texts, **kwargs):
    """Print formatted message

    Args:
        *texts (str): Texts to print. Each argument is rendered as paragraph.
        **kwargs: 'title' becomes coloured headline. exits=True performs sys
        exit.
    """
    exits = kwargs.get("exits", None)
    title = kwargs.get("title", None)
    title = "\033[93m{title}\033[0m\n".format(title=title) if title else ""
    message = "\n\n".join([text for text in texts])
    print("\n{title}{message}\n".format(title=title, message=message))
    if exits is not None:
        sys.exit(exits)


def get_json(url, desc):
    r = requests.get(url)
    if r.status_code != 200:
        raise OSError("%s: Received status code %s when fetching the resource"
                      % (desc, r.status_code))
    return r.json()



from __future__ import print_function, unicode_literals

import os
import subprocess
import sys

import plac

from snips_nlu import __about__
from snips_nlu.cli.link import link
from snips_nlu.cli.utils import get_json, prints
from snips_nlu.utils import get_package_path


# inspired from
# https://github.com/explosion/spaCy/blob/master/spacy/cli/download.py

@plac.annotations(
    resource_name=("Name of the language resources to download. Can be "
                   "either a shortcut, like 'en', or the full name of the "
                   "resources like 'snips_nlu_en'", "positional", None, str),
    pip_args=("Additional arguments to be passed to `pip install` when "
              "installing the model"))
def download(resource_name, *pip_args):
    """Download compatible resources for the specified language"""
    resource_name = resource_name.lower()
    shortcuts = get_json(__about__.__shortcuts__, "Resource shortcuts")
    full_resource_name = shortcuts.get(resource_name, resource_name)
    compatibility = _get_compatibility()
    version = _get_resources_version(full_resource_name, compatibility)
    dl = _download_model('{r}-{v}/{r}-{v}.tar.gz#egg={r}=={v}'
                         .format(r=full_resource_name, v=version), pip_args)
    if dl != 0:
        sys.exit(dl)
    try:
        # Get package path here because link uses
        # pip.get_installed_distributions() to check if model is a
        # package, which fails if model was just installed via
        # subprocess
        package_path = get_package_path(full_resource_name)
        link(full_resource_name, resource_name, force=True,
             resources_path=package_path)
    except:  # pylint:disable=bare-except
        prints(
            "Creating a shortcut link for '{r}' didn't work, but you can "
            "still load the resources via its full package name: "
            "snips_nlu.load_resources('{n}')".format(r=resource_name,
                                                     n=full_resource_name),
            title="Language resources were successfully downloaded, however "
                  "linking failed.")


def _get_compatibility():
    version = __about__.__version__
    table = get_json(__about__.__compatibility__, "Compatibility table")
    compatibility = table["snips-nlu"]
    if version not in compatibility:
        prints("No compatible resources found for version %s" % version,
               title="Resources compatibility error", exits=1)
    return compatibility[version]


def _get_resources_version(resource_name, compatibility):
    if resource_name not in compatibility:
        prints("No resources found for '%s'" % resource_name,
               title="Resources compatibility error", exits=1)
    return compatibility[resource_name][0]


def _download_model(filename, user_pip_args=None):
    download_url = __about__.__download_url__ + '/' + filename
    pip_args = ['--no-cache-dir', '--no-deps']
    if user_pip_args:
        pip_args.extend(user_pip_args)
    cmd = [sys.executable, '-m', 'pip', 'install'] + pip_args + [download_url]
    return subprocess.call(cmd, env=os.environ.copy())



from __future__ import unicode_literals

import errno
import importlib
import json
import numbers
import os
from builtins import object, str
from collections import OrderedDict, namedtuple, Mapping
from datetime import datetime
from pathlib import Path

import numpy as np
import pkg_resources

from snips_nlu.constants import (INTENTS, UTTERANCES, DATA, SLOT_NAME, ENTITY,
                                 END, START)

REGEX_PUNCT = {'\\', '.', '+', '*', '?', '(', ')', '|', '[', ']', '{', '}',
               '^', '$', '#', '&', '-', '~'}


class NotTrained(LookupError):
    """Exception used when a processing unit is used while not fitted"""
    pass


class ClassPropertyDescriptor(object):
    def __init__(self, fget, fset=None):
        self.fget = fget
        self.fset = fset

    def __get__(self, obj, klass=None):
        if klass is None:
            klass = type(obj)
        return self.fget.__get__(obj, klass)()

    def __set__(self, obj, value):
        if not self.fset:
            raise AttributeError("can't set attribute")
        type_ = type(obj)
        return self.fset.__get__(obj, type_)(value)

    def setter(self, func):
        if not isinstance(func, (classmethod, staticmethod)):
            func = classmethod(func)
        self.fset = func
        return self


def classproperty(func):
    if not isinstance(func, (classmethod, staticmethod)):
        func = classmethod(func)

    return ClassPropertyDescriptor(func)


# pylint: enable=C0103


def type_error(expected_type, found_type):
    return TypeError("Expected %s but found: %s" % (expected_type, found_type))


def validate_type(obj, expected_type):
    if not isinstance(obj, expected_type):
        raise type_error(expected_type=expected_type, found_type=type(obj))


def missing_key_error(key, object_label=None):
    if object_label is None:
        return KeyError("Missing key: '%s'" % key)
    return KeyError("Expected %s to have key: '%s'" % (object_label, key))


def validate_key(obj, key, object_label=None):
    if key not in obj:
        raise missing_key_error(key, object_label)


def validate_keys(obj, keys, object_label=None):
    for key in keys:
        validate_key(obj, key, object_label)


def validate_range(rng):
    if not isinstance(rng, (list, tuple)) or len(rng) != 2 or rng[0] > rng[1]:
        raise ValueError("range must be a length 2 list or tuple and must be "
                         "valid")


class LimitedSizeDict(OrderedDict):
    def __init__(self, *args, **kwds):
        if "size_limit" not in kwds:
            raise ValueError("'size_limit' must be passed as a keyword "
                             "argument")
        self.size_limit = kwds.pop("size_limit")
        if len(args) > 1:
            raise TypeError('expected at most 1 arguments, got %d' % len(args))
        if len(args) == 1 and len(args[0]) + len(kwds) > self.size_limit:
            raise ValueError("Tried to initialize LimitedSizedDict with more "
                             "value than permitted with 'limit_size'")
        super(LimitedSizeDict, self).__init__(*args, **kwds)

    def __setitem__(self, key, value, dict_setitem=OrderedDict.__setitem__):
        dict_setitem(self, key, value)
        self._check_size_limit()

    def _check_size_limit(self):
        if self.size_limit is not None:
            while len(self) > self.size_limit:
                self.popitem(last=False)

    def __eq__(self, other):
        if self.size_limit != other.size_limit:
            return False
        return super(LimitedSizeDict, self).__eq__(other)


class UnupdatableDict(dict):
    def __setitem__(self, key, value):
        if key in self:
            raise KeyError("Can't update key '%s'" % key)
        super(UnupdatableDict, self).__setitem__(key, value)


def namedtuple_with_defaults(typename, field_names, default_values=()):
    T = namedtuple(typename, field_names)  # pylint: disable=C0103
    T.__new__.__defaults__ = (None,) * len(T._fields)
    if isinstance(default_values, Mapping):
        prototype = T(**default_values)
    else:
        prototype = T(*default_values)
    T.__new__.__defaults__ = tuple(prototype)
    return T


def mkdir_p(path):
    """Reproduces the 'mkdir -p shell' command

    See
    http://stackoverflow.com/questions/600268/mkdir-p-functionality-in-python
    """
    try:
        os.makedirs(str(path))
    except OSError as exc:  # Python >2.5
        if exc.errno == errno.EEXIST and path.is_dir():
            pass
        else:
            raise


# pylint:disable=line-too-long
def regex_escape(s):
    """Escapes all regular expression meta characters in *s*

    The string returned may be safely used as a literal in a regular
    expression.

    This function is more precise than :func:`re.escape`, the latter escapes
    all non-alphanumeric characters which can cause cross-platform
    compatibility issues.

    References:

    - https://github.com/rust-lang/regex/blob/master/regex-syntax/src/lib.rs#L1685
    - https://github.com/rust-lang/regex/blob/master/regex-syntax/src/parser.rs#L1378
    """
    escaped_string = ""
    for c in s:
        if c in REGEX_PUNCT:
            escaped_string += "\\"
        escaped_string += c
    return escaped_string


# pylint:enable=line-too-long


def check_random_state(seed):
    """Turn seed into a :class:`numpy.random.RandomState` instance

    If seed is None, return the RandomState singleton used by np.random.
    If seed is an int, return a new RandomState instance seeded with seed.
    If seed is already a RandomState instance, return it.
    Otherwise raise ValueError.
    """
    # pylint: disable=W0212
    # pylint: disable=c-extension-no-member
    if seed is None or seed is np.random:
        return np.random.mtrand._rand  # pylint: disable=c-extension-no-member
    if isinstance(seed, (numbers.Integral, np.integer)):
        return np.random.RandomState(seed)
    if isinstance(seed, np.random.RandomState):
        return seed
    raise ValueError('%r cannot be used to seed a numpy.random.RandomState'
                     ' instance' % seed)


def get_slot_name_mapping(dataset, intent):
    """Returns a dict which maps slot names to entities for the provided intent
    """
    slot_name_mapping = dict()
    for utterance in dataset[INTENTS][intent][UTTERANCES]:
        for chunk in utterance[DATA]:
            if SLOT_NAME in chunk:
                slot_name_mapping[chunk[SLOT_NAME]] = chunk[ENTITY]
    return slot_name_mapping


def get_slot_name_mappings(dataset):
    """Returns a dict which maps intents to their slot name mapping"""
    return {intent: get_slot_name_mapping(dataset, intent)
            for intent in dataset[INTENTS]}


def ranges_overlap(lhs_range, rhs_range):
    if isinstance(lhs_range, dict) and isinstance(rhs_range, dict):
        return lhs_range[END] > rhs_range[START] \
               and lhs_range[START] < rhs_range[END]
    elif isinstance(lhs_range, (tuple, list)) \
            and isinstance(rhs_range, (tuple, list)):
        return lhs_range[1] > rhs_range[0] and lhs_range[0] < rhs_range[1]
    else:
        raise TypeError("Cannot check overlap on objects of type: %s and %s"
                        % (type(lhs_range), type(rhs_range)))


def elapsed_since(time):
    return datetime.now() - time


class DifferedLoggingMessage(object):

    def __init__(self, fn, *args, **kwargs):
        self.fn = fn
        self.args = args
        self.kwargs = kwargs

    def __str__(self):
        return str(self.fn(*self.args, **self.kwargs))


def json_debug_string(dict_data):
    return json.dumps(dict_data, ensure_ascii=False, indent=2, sort_keys=True)


def log_elapsed_time(logger, level, output_msg=None):
    if output_msg is None:
        output_msg = "Elapsed time ->:\n{elapsed_time}"

    def get_wrapper(fn):
        def wrapped(*args, **kwargs):
            start = datetime.now()
            msg_fmt = dict()
            res = fn(*args, **kwargs)
            if "elapsed_time" in output_msg:
                msg_fmt["elapsed_time"] = datetime.now() - start
            logger.log(level, output_msg.format(**msg_fmt))
            return res

        return wrapped

    return get_wrapper


def log_result(logger, level, output_msg=None):
    if output_msg is None:
        output_msg = "Result ->:\n{result}"

    def get_wrapper(fn):
        def wrapped(*args, **kwargs):
            msg_fmt = dict()
            res = fn(*args, **kwargs)
            if "result" in output_msg:
                try:
                    res_debug_string = json_debug_string(res)
                except TypeError:
                    res_debug_string = str(res)
                msg_fmt["result"] = res_debug_string
            logger.log(level, output_msg.format(**msg_fmt))
            return res

        return wrapped

    return get_wrapper


def is_package(name):
    """Check if name maps to a package installed via pip.

    Args:
        name (str): Name of package

    Returns:
        bool: True if an installed packaged corresponds to this name, False
            otherwise.
    """
    name = name.lower().replace("-", "_")
    packages = pkg_resources.working_set.by_key.keys()
    for package in packages:
        if package.lower().replace("-", "_") == name:
            return True
    return False


def get_package_path(name):
    """Get the path to an installed package.

    Args:
        name (str): Package name

    Returns:
        class:`.Path`: Path to the installed package
    """
    name = name.lower().replace("-", "_")
    pkg = importlib.import_module(name)
    return Path(pkg.__file__).parent



import io
import os

from setuptools import setup, find_packages

packages = [p for p in find_packages()
            if "tests" not in p and "debug" not in p]

root = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(root, "snips_nlu", "__about__.py"),
             encoding="utf8") as f:
    about = dict()
    exec(f.read(), about)


with io.open(os.path.join(root, "README.rst"), encoding="utf8") as f:
    readme = f.read()

nlu_metrics_version = "0.12.0"

required = [
    "enum34==1.1.6",
    "pathlib==1.0.1",
    "numpy==1.14.0",
    "scipy==1.0.0",
    "scikit-learn==0.19.1",
    "sklearn-crfsuite==0.3.6",
    "semantic_version==2.6.0",
    "snips_nlu_utils==0.6.1",
    "snips_nlu_ontology==0.57.1",
    "num2words==0.5.6",
    "plac==0.9.6",
    "requests==2.18.4"
]

extras_require = {
    "doc": [
        "sphinx==1.7.1",
        "sphinxcontrib-napoleon==0.6.1",
        "sphinx-rtd-theme==0.2.4"
    ],
    "metrics": [
        "snips_nlu_metrics==%s" % nlu_metrics_version,
    ],
    "test": [
        "mock==2.0.0",
        "snips_nlu_metrics==%s" % nlu_metrics_version,
        "pylint==1.8.2",
        "coverage==4.4.2"
    ],
    "integration_test": [
        "snips_nlu_metrics==%s" % nlu_metrics_version,
    ]
}

setup(name=about["__title__"],
      description=about["__summary__"],
      long_description=readme,
      version=about["__version__"],
      author=about["__author__"],
      author_email=about["__email__"],
      license=about["__license__"],
      url=about["__uri__"],
      install_requires=required,
      extras_require=extras_require,
      classifiers=[
          "Programming Language :: Python :: 2",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.4",
          "Programming Language :: Python :: 3.5",
          "Programming Language :: Python :: 3.6",
      ],
      packages=packages,
      include_package_data=True,
      entry_points={
          "console_scripts": [
              "snips-nlu=snips_nlu.__main__:main"
          ]
      },
      zip_safe=False)



from __future__ import print_function, unicode_literals

from pathlib import Path

import plac

from snips_nlu.cli.compatibility import create_symlink
from snips_nlu.cli.utils import prints
from snips_nlu.constants import DATA_PATH
from snips_nlu.utils import is_package, get_package_path


# inspired from
# https://github.com/explosion/spaCy/blob/master/spacy/cli/link.py

@plac.annotations(
    origin=("package name or local path to model", "positional", None, str),
    link_name=("name of shortcut link to create", "positional", None, str),
    force=("force overwriting of existing link", "flag", "f", bool))
def link(origin, link_name, force=False, resources_path=None):
    """
    Create a symlink for language resources within the snips_nlu/data
    directory. Accepts either the name of a pip package, or the local path to
    the resources data directory.

    Linking resources allows loading them via
    snips_nlu.load_resources(link_name).
    """
    if is_package(origin):
        resources_path = get_package_path(origin)
    else:
        resources_path = Path(origin) if resources_path is None \
            else Path(resources_path)
    if not resources_path.exists():
        raise OSError("%s not found" % str(resources_path))
    link_path = DATA_PATH / link_name
    if link_path.is_symlink() and not force:
        raise OSError("Symlink already exists: %s" % str(link_path))
    elif link_path.is_symlink():
        link_path.unlink()
    elif link_path.exists():
        raise OSError("Symlink cannot be overwritten: %s" % str(link_path))
    create_symlink(link_path, resources_path)
    prints("%s --> %s" % (str(resources_path), str(link_path)),
           "You can now load the resources via snips_nlu.load_resources('%s')"
           % link_name, title="Linking successful")
